---------------------------------------------------------------
--                                                           
--                                                           
--  FORMAT.ADB 
--  Description : Formats an Ada program with indenting/capitalization 
--                       
--  By: James Hopper
--      Michael B. Feldman
--      Martin C. Carlisle
--
--  Copyright (C) 1997 James Hopper, Michael B. Feldman
--  $Revision: 1.3 $
--  $Date: 1998-06-22 16:35:43-06 $
--
--  $Revision: 2.0 $
--  $Date: 1999-05-24 13:00:00-06 $
--                                         
-- FORMAT is free software; you can redistribute it and/or
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly
-- indicate if it has been modified.
--
-- FORMAT is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
---------------------------------------------------------------
-- Change log:
-- 09/16/99 (mcc) : solved LF/CR problem, by replacing Direct_IO
--                  with Text_IO
-- 09/16/99 (mcc) : added flags for Advanced and Types
---------------------------------------------------------------

with Ada.Text_IO;
with Ada.Command_Line;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Settings;
with Lexer;
with reformat_pkg;

procedure Reformat is

  The_File    : Ada.Text_IO.File_Type;
  Input_File  : Ada.Text_IO.File_Type;
  File_Length : Integer;
  File_Count  : Integer;

  File_Text     : Lexer.Stringpointer;
  New_File_Text : Lexer.Stringpointer;
  Temp_Character: Character;
  Success       : Boolean;
  
  Command       : Ada.Strings.Unbounded.Unbounded_String;
  
  Index    : Integer;

begin

  if Ada.Command_Line.Argument_Count >= 1 then
    Index:= 1;
    loop
      Command:= Ada.Strings.Unbounded.To_Unbounded_String(
        Ada.Command_Line.Argument(Index));
      exit when Ada.Strings.Unbounded.Element(Command,1) /= '-';
      if Command = "-id" then -- identifier case
        Index:= Index + 1;
        Settings.Set_Identifier_Case(
           Settings.Text_Case'value(Ada.Command_Line.Argument(Index)));
      elsif Command = "-rw" then -- reserved word case
        Index:= Index + 1;
        Settings.Set_Keyword_Case(
           Settings.Text_Case'value(Ada.Command_Line.Argument(Index)));
      elsif Command = "-c" then -- colorize
        Index:= Index + 1;
        Settings.Set_Colorization(
           Settings.Colorize_Option'value(Ada.Command_Line.Argument(Index)));
      elsif Command = "-i" then -- indent
        Index:= Index + 1;
        Settings.Set_Normal_Indent(
           Natural'value(Ada.Command_Line.Argument(Index)));
      elsif Command = "-nip" then -- don't indent parenthesis
        Settings.Set_Indent_Nested_Paren(True);
      elsif Command = "-nar" then -- don't do advanced
        Settings.Set_Do_Vertical_Spacing(False);
      elsif Command = "-nrt" then -- don't reformat types
        Settings.Set_Reformat_Types(False);
      end if;
      exit when Index = Ada.Command_Line.Argument_Count;
      Index:= Index + 1;
    end loop;

    If Index <= Ada.Command_Line.Argument_Count then -- if no file name don't do anything

      Ada.Text_IO.Open(
        Name => Ada.Command_Line.Argument(Index),
        File => Input_File,
        Mode => Ada.Text_IO.In_File);

      Index:= Index + 1;

      File_Length:= 0;
      
      while not Ada.Text_IO.End_Of_File(Input_File) loop
         while not Ada.Text_IO.End_Of_Line(Input_File) loop
            Ada.Text_Io.Get(Input_File,Temp_Character);
            File_Length := File_Length + 1;         
         end loop;
         if not Ada.Text_IO.End_Of_File(Input_File) then
            Ada.Text_IO.Skip_Line(Input_File);
            File_Length := File_Length + 2;
         end if;
      end loop;
      
      File_Text:= new String(1..File_Length);
      
      Ada.Text_IO.Reset(Input_File);
      
      File_Count := 1;
      
      while not Ada.Text_IO.End_Of_File(Input_File) loop
         while not Ada.Text_IO.End_Of_Line(Input_File) loop
            Ada.Text_Io.Get(Input_File,File_Text(File_Count));
            File_Count := File_Count + 1;         
         end loop;
         if not Ada.Text_IO.End_Of_File(Input_File) then
            Ada.Text_IO.Skip_Line(Input_File);
            File_Text(File_Count) := ASCII.CR;
            File_Text(File_Count+1) := ASCII.LF;
            File_Count := File_Count + 2;
         end if;
      end loop;
      
      Ada.Text_IO.Close(Input_File);

      reformat_pkg.Reformat(The_String => File_Text,
        Success            => Success,
        Result             => New_File_Text);

      if not Success then
        Ada.Text_IO.Put_Line(
          "Exception raised.  Probably your program is badly formed.");
      end if;

      if Index <= Ada.Command_Line.Argument_Count then -- if output filename use it

        Ada.Text_IO.Create(
          Name => Ada.Command_Line.Argument(Index),
          Mode => Ada.Text_IO.Out_File,
          File => The_File);
        Index:= Index + 1;

        for i in New_File_Text'First..New_File_Text'Last loop
          case New_File_Text(i) is
             when ASCII.LF =>
                Ada.Text_IO.New_Line(The_File);
             -- might be null terminated (if RTF?), so only
             -- output character if it is not null.
             when ASCII.CR|Character'First =>
                null;
             when others =>
                Ada.Text_IO.Put(File => The_File,
                  Item => New_File_Text(i));
          end case;
        end loop;


        Ada.Text_IO.Close(The_File);

      else -- otherwise write to standard out
        -- don't sweat the LF/CR stuff here
        Ada.Text_IO.put_line(New_File_Text.all);
      end if;

    else
      Ada.Text_IO.put_line("Need Filename");
    end if;
  else
    Ada.Text_IO.put_line("Usage " & Ada.Command_Line.Command_Name &
      " [options] infile [outfile]");
    Ada.Text_IO.put_line("options =");
    Ada.Text_IO.put_line(
      "   -id Upper_Case|Lower_Case|Mixed_No_Force_Lower|Mixed_Force_Lower");
    Ada.Text_IO.put_line(
      "   -rw Upper_Case|Lower_Case|Mixed_No_Force_Lower|Mixed_Force_Lower");
    Ada.Text_IO.put_line(
      "   -c Colorize_Only|Colorize|Bold|Bold_Only|No_Colorize");
    Ada.Text_IO.put_line("   -i N");
    Ada.Text_IO.put_line("   -nip");
    Ada.Text_IO.new_line;
    Ada.Text_IO.put_line("-id specifies case for identifiers");
    Ada.Text_IO.put_line("-rw specifies case for reserved words");
    Ada.Text_IO.put_line("-i specifies amount to indent each level");
    Ada.Text_IO.put_line("-nip specifies not to indent based on parenthesis depth");
    Ada.Text_IO.put_line("-nar specifies not to do advanced reformat");
    Ada.Text_IO.put_line("-nrt specifies not to reformat types");
    Ada.Text_IO.put_line(
      "-c specifies whether to output colorized or bolded RTF, or plain ASCII");
  end if;

end Reformat;
